package io.gitlab.jfronny.commons.test;

import io.gitlab.jfronny.commons.ComparableVersion;
import io.gitlab.jfronny.commons.serialize.Serializer;
import io.gitlab.jfronny.commons.serialize.gson.api.Ignore;
import io.gitlab.jfronny.commons.serialize.gson.api.v1.GsonHolders;
import org.junit.jupiter.api.BeforeAll;
import org.junit.jupiter.api.Test;

import java.io.IOException;

import static org.junit.jupiter.api.Assertions.*;

public class GsonTest {
    @BeforeAll
    static void setup() {
        GsonHolders.registerSerializer();
    }

    @Test
    void gsonIgnore() throws IOException {
        assertEquals("null", Serializer.getInstance().serialize(new ExampleTypeHidden()));
        assertEquals("{\"shouldShow\":\"Yes\"}", Serializer.getInstance().serialize(new TransitiveHiddenType()));
        assertEquals("{\"id\":\"Yes\",\"shown\":{\"shouldShow\":\"Yes\"}}", Serializer.getInstance().serialize(new ExampleType()));
    }

    @Test
    void comparableVersionAdapter() throws IOException {
        assertEquals("\"1.0.0\"", Serializer.getInstance().serialize(new ComparableVersion("1.0.0")));
        assertDoesNotThrow(() -> assertEquals(new ComparableVersion("1.0.0"), Serializer.getInstance().deserialize("\"1.0.0\"", ComparableVersion.class)));
    }

    public static class ExampleType {
        String id = "Yes";
        @Ignore
        String other = "No";
        transient Integer one = 1;
        ExampleTypeHidden type = new ExampleTypeHidden();
        TransitiveHiddenType shown = new TransitiveHiddenType();
    }

    @Ignore
    public static class ExampleTypeHidden {
        String someMetadata = "No";
    }

    public static class TransitiveHiddenType extends ExampleTypeHidden {
        String shouldShow = "Yes";
    }
}
