package io.gitlab.jfronny.muscript.ast.bool;

import io.gitlab.jfronny.muscript.data.Scope;
import io.gitlab.jfronny.muscript.ast.BoolExpr;
import io.gitlab.jfronny.muscript.ast.Expr;
import io.gitlab.jfronny.muscript.ast.literal.BoolLiteral;

public class And extends BoolExpr {
    private final BoolExpr left;
    private final BoolExpr right;

    public And(int chStart, int chEnd, BoolExpr left, BoolExpr right) {
        super(chStart, chEnd);
        this.left = left;
        this.right = right;
    }

    @Override
    public Boolean get(Scope dataRoot) {
        return left.get(dataRoot) && right.get(dataRoot);
    }

    @Override
    public BoolExpr optimize() {
        BoolExpr left = this.left.optimize();
        BoolExpr right = this.right.optimize();
        if (left instanceof BoolLiteral literal) return literal.value ? right : Expr.literal(chStart, chEnd, false);
        if (right instanceof BoolLiteral literal) return literal.value ? left : Expr.literal(chStart, chEnd, false);
        return new And(chStart, chEnd, left, right);
    }

    @Override
    public boolean equals(Object obj) {
        return obj instanceof And and && left.equals(and.left) && right.equals(and.right);
    }
}
