package io.gitlab.jfronny.muscript.ast.bool;

import io.gitlab.jfronny.muscript.data.Scope;
import io.gitlab.jfronny.muscript.ast.BoolExpr;
import io.gitlab.jfronny.muscript.ast.Expr;
import io.gitlab.jfronny.muscript.ast.literal.BoolLiteral;

public class Or extends BoolExpr {
    private final BoolExpr left;
    private final BoolExpr right;

    public Or(int chStart, int chEnd, BoolExpr left, BoolExpr right) {
        super(chStart, chEnd);
        this.left = left;
        this.right = right;
    }

    @Override
    public Boolean get(Scope dataRoot) {
        return left.get(dataRoot) || right.get(dataRoot);
    }

    @Override
    public BoolExpr optimize() {
        BoolExpr left = this.left.optimize();
        BoolExpr right = this.right.optimize();
        if (left instanceof BoolLiteral literal) return literal.value ? Expr.literal(chStart, chEnd, true) : right;
        if (right instanceof BoolLiteral literal) return literal.value ? Expr.literal(chStart, chEnd, true) : left;
        return new Or(chStart, chEnd, left, right);
    }

    @Override
    public boolean equals(Object obj) {
        return obj instanceof Or or && left.equals(or.left) && right.equals(or.right);
    }
}
