package io.gitlab.jfronny.muscript.ast.conditional;

import io.gitlab.jfronny.muscript.data.Scope;
import io.gitlab.jfronny.muscript.ast.BoolExpr;
import io.gitlab.jfronny.muscript.ast.StringExpr;
import io.gitlab.jfronny.muscript.ast.literal.BoolLiteral;

public class StringConditional extends StringExpr {
    public final BoolExpr condition;
    public final StringExpr trueExpr;
    public final StringExpr falseExpr;

    public StringConditional(int chStart, int chEnd, BoolExpr condition, StringExpr trueExpr, StringExpr falseExpr) {
        super(chStart, chEnd);
        this.condition = condition;
        this.trueExpr = trueExpr;
        this.falseExpr = falseExpr;
    }

    @Override
    public String get(Scope dataRoot) {
        return condition.get(dataRoot) ? trueExpr.get(dataRoot) : falseExpr.get(dataRoot);
    }

    @Override
    public StringExpr optimize() {
        BoolExpr condition = this.condition.optimize();
        StringExpr trueExpr = this.trueExpr.optimize();
        StringExpr falseExpr = this.falseExpr.optimize();
        if (condition instanceof BoolLiteral literal) return literal.value ? trueExpr : falseExpr;
        if (trueExpr.equals(falseExpr)) return trueExpr;
        return new StringConditional(chStart, chEnd, condition, trueExpr, falseExpr);
    }

    @Override
    public boolean equals(Object obj) {
        return obj instanceof StringConditional conditional
                && condition.equals(conditional.condition)
                && trueExpr.equals(conditional.trueExpr)
                && falseExpr.equals(conditional.falseExpr);
    }
}
