package io.gitlab.jfronny.muscript.ast.dynamic;

import io.gitlab.jfronny.muscript.data.Scope;
import io.gitlab.jfronny.muscript.data.dynamic.Dynamic;
import io.gitlab.jfronny.muscript.ast.DynamicExpr;
import io.gitlab.jfronny.muscript.annotations.CanThrow;
import io.gitlab.jfronny.muscript.annotations.UncheckedDynamic;
import io.gitlab.jfronny.muscript.error.LocationalException;

@CanThrow
@UncheckedDynamic
public class Variable extends DynamicExpr {
    private final String name;

    public Variable(int chStart, int chEnd, String name) {
        super(chStart, chEnd);
        this.name = name;
    }

    @Override
    public Dynamic<?> get(Scope dataRoot) {
        if (dataRoot.asObject().has(name)) return dataRoot.asObject().get(name);
        else if (name.contains("::")) {
            Dynamic<?> res = dataRoot;
            for (String s : name.split("::")) {
                if (!res.asObject().has(s))
                    throw new LocationalException(chStart, chEnd, "This object doesn't contain '" + name + "'");
                res = res.asObject().get(s);
            }
            return res;
        } else throw new LocationalException(chStart, chEnd, "This object doesn't contain '" + name + "'");
    }

    @Override
    public DynamicExpr optimize() {
        return this;
    }

    @Override
    public boolean equals(Object obj) {
        return obj instanceof Variable variable && name.equals(variable.name);
    }
}
