package io.gitlab.jfronny.muscript.ast.dynamic.unpack;

import io.gitlab.jfronny.muscript.data.Scope;
import io.gitlab.jfronny.muscript.data.dynamic.DynamicTypeConversionException;
import io.gitlab.jfronny.muscript.ast.DynamicExpr;
import io.gitlab.jfronny.muscript.ast.StringExpr;
import io.gitlab.jfronny.muscript.annotations.CanThrow;
import io.gitlab.jfronny.muscript.annotations.UncheckedDynamic;
import io.gitlab.jfronny.muscript.ast.dynamic.DynamicCoerce;

@CanThrow
@UncheckedDynamic
public class StringUnpack extends StringExpr {
    public final DynamicExpr inner;

    public StringUnpack(int chStart, int chEnd, DynamicExpr inner) {
        super(chStart, chEnd);
        this.inner = inner;
    }

    @Override
    public String get(Scope dataRoot) {
        try {
            return inner.get(dataRoot).asString().getValue();
        } catch (DynamicTypeConversionException e) {
            throw e.locational(chStart, chEnd);
        }
    }

    @Override
    public StringExpr optimize() {
        DynamicExpr inner = this.inner.optimize();
        if (inner instanceof DynamicCoerce coerce) return coerce.inner.asStringExpr();
        return new StringUnpack(chStart, chEnd, inner);
    }

    @Override
    public boolean equals(Object obj) {
        return obj instanceof StringUnpack unpack && inner.equals(unpack.inner);
    }
}
