package io.gitlab.jfronny.muscript.data;

import io.gitlab.jfronny.commons.data.ImmCollection;
import io.gitlab.jfronny.muscript.data.dynamic.*;

import java.util.*;
import java.util.function.Function;

public class Scope implements DObject {
    private final DObject source;
    private final Map<String, Dynamic<?>> override = new HashMap<>();

    public Scope() {
        this(DFinal.of(Map.of()));
    }

    public Scope(DObject source) {
        this.source = source;
    }

    @Override
    public Map<String, Dynamic<?>> getValue() {
        var map = new HashMap<>(source.getValue());
        map.putAll(override);
        return ImmCollection.of(map);
    }

    /**
     * @deprecated Using the convenience methods is recommended wherever possible.
     */
    @Deprecated(forRemoval = false)
    public Scope set(String key, Dynamic<?> value) {
        override.put(key, value);
        return this;
    }

    public Scope set(String key, boolean value) {
        return set(key, DFinal.of(value));
    }

    public Scope set(String key, double value) {
        return set(key, DFinal.of(value));
    }

    public Scope set(String key, String value) {
        return set(key, DFinal.of(value));
    }

    public Scope set(String key, Map<String, Dynamic<?>> value) {
        return set(key, DFinal.of(value));
    }

    public Scope set(String key, List<Dynamic<?>> value) {
        return set(key, DFinal.of(value));
    }

    public Scope set(String key, Function<DList, Dynamic<?>> value) {
        return set(key, DFinal.of(value));
    }

    public Scope fork() {
        return new Scope(this);
    }
}
