package io.gitlab.jfronny.muscript.data.dynamic;

import java.util.*;

/**
 * An enum represented as an OObject with automatic conversions to a number (index) and string (selected value)
 */
public record DEnum(Map<String, Dynamic<?>> values, DEnumEntry value) implements DObject {
    public DEnum(List<String> values, String value) {
        this(createMap(values, value), new DEnumEntry(value, values.indexOf(value), true));
    }

    @Override
    public Map<String, Dynamic<?>> getValue() {
        return values;
    }

    @Override
    public DString asString() {
        return value;
    }

    @Override
    public DNumber asNumber() {
        return value.asNumber();
    }

    private static Map<String, Dynamic<?>> createMap(List<String> values, String value) {
        Map<String, Dynamic<?>> result = new LinkedHashMap<>();
        DEnumEntry v = new DEnumEntry(value, values.indexOf(value), true);
        result.put("value", v);
        for (int i = 0; i < values.size(); i++) {
            result.put(values.get(i), new DEnumEntry(values.get(i), i, values.get(i).equals(value)));
        }
        return Map.copyOf(result);
    }

    public record DEnumEntry(String value, int index, boolean selected) implements DString {
        @Override
        public String getValue() {
            return value;
        }

        @Override
        public DNumber asNumber() {
            return DFinal.of(index);
        }

        @Override
        public DBool asBool() {
            return DFinal.of(selected);
        }
    }
}
