package io.gitlab.jfronny.muscript.data.dynamic;

import io.gitlab.jfronny.commons.StringFormatter;
import io.gitlab.jfronny.commons.data.ImmCollection;

import java.util.List;
import java.util.Map;
import java.util.function.Function;

public record DFinal<T>(T value) implements Dynamic<T> {
    public static DBool of(boolean b) {
        return new FBool(b);
    }

    public static DNumber of(double b) {
        return new FNumber(b);
    }

    public static DString of(String b) {
        return new FString(b);
    }

    public static DObject of(Map<String, Dynamic<?>> b) {
        return new FObject(Map.copyOf(b));
    }

    public static DList of(Dynamic<?>... b) {
        return new FList(List.of(b));
    }

    public static DList of(List<Dynamic<?>> b) {
        return new FList(ImmCollection.of(b));
    }

    public static DCallable of(Function<DList, Dynamic<?>> b) {
        return new FCallable(b);
    }

    @Override
    public T getValue() {
        return value;
    }

    @Override
    public String toString() {
        return StringFormatter.toString(value);
    }

    private record FBool(boolean value) implements DBool {
        @Override
        public Boolean getValue() {
            return value;
        }

        @Override
        public String toString() {
            return StringFormatter.toString(value);
        }
    }

    private record FNumber(double value) implements DNumber {
        @Override
        public Double getValue() {
            return value;
        }

        @Override
        public String toString() {
            return StringFormatter.toString(value);
        }
    }

    private record FString(String value) implements DString {
        @Override
        public String getValue() {
            return value;
        }

        @Override
        public String toString() {
            return StringFormatter.toString(value);
        }
    }

    private record FObject(Map<String, Dynamic<?>> value) implements DObject {
        @Override
        public Map<String, Dynamic<?>> getValue() {
            return value;
        }

        @Override
        public String toString() {
            return StringFormatter.toString(value);
        }
    }

    private record FList(List<Dynamic<?>> value) implements DList {
        @Override
        public List<Dynamic<?>> getValue() {
            return value;
        }

        @Override
        public String toString() {
            return StringFormatter.toString(value);
        }
    }

    private record FCallable(Function<DList, Dynamic<?>> value) implements DCallable {
        @Override
        public Function<DList, Dynamic<?>> getValue() {
            return value;
        }

        @Override
        public String toString() {
            return "<Callable>";
        }
    }
}
