package io.gitlab.jfronny.muscript.error;

/**
 * An exception type with a location
 * For use in MuScript, can be converted to a pretty LocationalError with asPrintable
 */
public class LocationalException extends RuntimeException {
    private final int start, end;

    public LocationalException(int start, int end) {
        super();
        this.start = start;
        this.end = end;
    }

    public LocationalException(int start, int end, String message) {
        super(message);
        this.start = start;
        this.end = end;
    }

    public LocationalException(int start, int end, String message, Throwable cause) {
        super(message, cause);
        this.start = start;
        this.end = end;
    }

    public LocationalException(int start, int end, Throwable cause) {
        super(cause);
        this.start = start;
        this.end = end;
    }

    public LocationalError asPrintable(String source) {
        return LocationalError.create(source, start, end, getLocalizedMessage());
    }
}
