package io.gitlab.jfronny.commons.log;

import org.jetbrains.annotations.Nullable;

import java.util.Objects;

public class DelegateLogger implements Logger {
    public DelegateLogger() {
        this(null);
    }

    public DelegateLogger(Logger delegate) {
        this.delegate = delegate;
    }

    protected Logger delegate;
    protected Level minimumLevel = Level.TRACE;

    @Override
    public @Nullable String getName() {
        return delegate.getName();
    }

    @Override
    public void trace(String msg) {
        if (minimumLevel.shouldOmit(Level.TRACE)) return;
        delegate.trace(msg);
    }

    @Override
    public void trace(String format, Object arg) {
        if (minimumLevel.shouldOmit(Level.TRACE)) return;
        delegate.trace(format, arg);
    }

    @Override
    public void trace(String format, Object... args) {
        if (minimumLevel.shouldOmit(Level.TRACE)) return;
        delegate.trace(format, args);
    }

    @Override
    public void trace(String msg, Throwable t) {
        if (minimumLevel.shouldOmit(Level.TRACE)) return;
        delegate.trace(msg, t);
    }

    @Override
    public void debug(String msg) {
        if (minimumLevel.shouldOmit(Level.DEBUG)) return;
        delegate.debug(msg);
    }

    @Override
    public void debug(String format, Object arg) {
        if (minimumLevel.shouldOmit(Level.DEBUG)) return;
        delegate.debug(format, arg);
    }

    @Override
    public void debug(String format, Object... args) {
        if (minimumLevel.shouldOmit(Level.DEBUG)) return;
        delegate.debug(format, args);
    }

    @Override
    public void debug(String msg, Throwable t) {
        if (minimumLevel.shouldOmit(Level.DEBUG)) return;
        delegate.debug(msg, t);
    }

    @Override
    public void info(String msg) {
        if (minimumLevel.shouldOmit(Level.INFO)) return;
        delegate.info(msg);
    }

    @Override
    public void info(String format, Object arg) {
        if (minimumLevel.shouldOmit(Level.INFO)) return;
        delegate.info(format, arg);
    }

    @Override
    public void info(String format, Object... args) {
        if (minimumLevel.shouldOmit(Level.INFO)) return;
        delegate.info(format, args);
    }

    @Override
    public void info(String msg, Throwable t) {
        if (minimumLevel.shouldOmit(Level.INFO)) return;
        delegate.info(msg, t);
    }

    @Override
    public void warn(String msg) {
        if (minimumLevel.shouldOmit(Level.WARN)) return;
        delegate.warn(msg);
    }

    @Override
    public void warn(String format, Object arg) {
        if (minimumLevel.shouldOmit(Level.WARN)) return;
        delegate.warn(format, arg);
    }

    @Override
    public void warn(String format, Object... args) {
        if (minimumLevel.shouldOmit(Level.WARN)) return;
        delegate.warn(format, args);
    }

    @Override
    public void warn(String msg, Throwable t) {
        if (minimumLevel.shouldOmit(Level.WARN)) return;
        delegate.warn(msg, t);
    }

    @Override
    public void error(String msg) {
        if (minimumLevel.shouldOmit(Level.ERROR)) return;
        delegate.error(msg);
    }

    @Override
    public void error(String format, Object arg) {
        if (minimumLevel.shouldOmit(Level.ERROR)) return;
        delegate.error(format, arg);
    }

    @Override
    public void error(String format, Object... args) {
        if (minimumLevel.shouldOmit(Level.ERROR)) return;
        delegate.error(format, args);
    }

    @Override
    public void error(String msg, Throwable t) {
        if (minimumLevel.shouldOmit(Level.ERROR)) return;
        delegate.error(msg, t);
    }

    @Override
    public String format(String format, Object arg) {
        return delegate.format(format, arg);
    }

    @Override
    public String format(String format, Object... args) {
        return delegate.format(format, args);
    }

    @Override
    public String format(String msg, Throwable t) {
        return delegate.format(msg, t);
    }

    public Logger getDelegate() {
        return delegate;
    }

    /**
     * Set the minimum level messages must have to not be discarded
     *
     * @param minimumLevel The new minimum
     */
    public void setMinimumLevel(Level minimumLevel) {
        this.minimumLevel = Objects.requireNonNull(minimumLevel);
    }

    /**
     * Get the minimum level messages must have to not be discarded
     *
     * @return the minimum
     */
    public Level getMinimumLevel() {
        return minimumLevel;
    }
}
