package io.gitlab.jfronny.commons.log;

import java.time.LocalTime;
import java.time.format.DateTimeFormatter;

public class StdoutLogger implements Logger {
    private final String name;
    private final boolean color;
    private final boolean thread;
    private final boolean timestamp;

    public StdoutLogger(String name) {
        this(name, false);
    }

    public StdoutLogger(String name, boolean color) {
        this(name, color, false, false);
    }

    public StdoutLogger(String name, boolean color, boolean thread, boolean timestamp) {
        this.name = name;
        this.color = color;
        this.thread = thread;
        this.timestamp = timestamp;
    }

    public static StdoutLogger fancy(String name) {
        return new StdoutLogger(name, true, true, true);
    }

    @Override
    public String getName() {
        return name;
    }

    @Override
    public void trace(String msg) {
        System.out.println(generateMessage(msg, Level.TRACE));
    }

    @Override
    public void debug(String msg) {
        System.out.println(generateMessage(msg, Level.DEBUG));
    }

    @Override
    public void info(String msg) {
        System.out.println(generateMessage(msg, Level.INFO));
    }

    @Override
    public void warn(String msg) {
        System.out.println(generateMessage(msg, Level.WARN));
    }

    @Override
    public void error(String msg) {
        System.out.println(generateMessage(msg, Level.ERROR));
    }

    private String generateMessage(String msg, Level level) {
        StringBuilder sb = new StringBuilder();
        // Timestamp
        if (this.timestamp) {
            sb.append(DateTimeFormatter.ofPattern("HH:mm:ss.SSS").format(LocalTime.now())).append(' ');
        }
        // Thread name
        if (this.thread) {
            String thread = Thread.currentThread().getName();
            if (this.color) {
                sb.append(OutputColors.CYAN_BOLD).append(thread).append(OutputColors.RESET).append(' ');
            } else {
                sb.append('{').append(thread).append("} ");
            }
        }
        // Logger name
        if (this.color) {
            sb.append(OutputColors.GREEN_BOLD).append(name).append(OutputColors.RESET).append(' ');
        } else {
            sb.append('(').append(name).append(") ");
        }
        // Level
        if (this.color) {
            sb.append(level.color).append(level.name()).append(OutputColors.RESET).append(' ');
        } else {
            sb.append(level.name().charAt(0)).append(' ');
        }
        // Message
        sb.append(msg);
        return sb.toString();
    }
}
