package io.gitlab.jfronny.commons.reflect;

import java.lang.invoke.*;
import java.util.function.*;

@SuppressWarnings("unchecked")
public class LambdaFactory {
    private static final MethodHandles.Lookup LOOKUP_ROOT = MethodHandles.lookup();

    public static MethodHandles.Lookup lookup(Class<?> klazz) throws IllegalAccessException {
        return MethodHandles.privateLookupIn(klazz, LOOKUP_ROOT);
    }

    public static Runnable runnable(MethodHandles.Lookup lookup, MethodHandle handle) throws Throwable {
        return (Runnable) LambdaMetafactory.metafactory(
                lookup,
                "run",
                MethodType.methodType(Runnable.class),
                MethodType.methodType(Void.TYPE),
                handle,
                MethodType.methodType(Void.TYPE)
        ).getTarget().invoke();
    }

    public static <TIn> Consumer<TIn> consumer(MethodHandles.Lookup lookup, MethodHandle handle, Class<TIn> parameterType) throws Throwable {
        return (Consumer<TIn>) LambdaMetafactory.metafactory(
                lookup,
                "accept",
                MethodType.methodType(Consumer.class),
                MethodType.methodType(Void.TYPE, Object.class),
                handle,
                MethodType.methodType(Void.TYPE, parameterType)
        ).getTarget().invoke();
    }

    public static <TIn1, TIn2> BiConsumer<TIn1, TIn2> consumer(MethodHandles.Lookup lookup, MethodHandle handle, Class<TIn1> parameterType1, Class<TIn2> parameterType2) throws Throwable {
        return (BiConsumer<TIn1, TIn2>) LambdaMetafactory.metafactory(
                lookup,
                "accept",
                MethodType.methodType(BiConsumer.class),
                MethodType.methodType(Void.TYPE, Object.class, Object.class),
                handle,
                MethodType.methodType(Void.TYPE, parameterType1, parameterType2)
        ).getTarget().invoke();
    }

    public static <TOut> Supplier<TOut> supplier(MethodHandles.Lookup lookup, MethodHandle handle, Class<TOut> returnType) throws Throwable {
        return (Supplier<TOut>) LambdaMetafactory.metafactory(
                lookup,
                "get",
                MethodType.methodType(Supplier.class),
                MethodType.methodType(Object.class),
                handle,
                MethodType.methodType(returnType)
        ).getTarget().invoke();
    }

    public static <TIn, TOut> Function<TIn, TOut> function(MethodHandles.Lookup lookup, MethodHandle handle, Class<TOut> returnType, Class<TIn> parameterType) throws Throwable {
        return (Function<TIn, TOut>) LambdaMetafactory.metafactory(
                lookup,
                "apply",
                MethodType.methodType(Function.class),
                MethodType.methodType(Object.class, Object.class),
                handle,
                MethodType.methodType(returnType, parameterType)
        ).getTarget().invoke();
    }

    public static <TIn1, TIn2, TOut> BiFunction<TIn1, TIn2, TOut> function(MethodHandles.Lookup lookup, MethodHandle handle, Class<TOut> returnType, Class<TIn1> parameterType1, Class<TIn2> parameterType2) throws Throwable {
        return (BiFunction<TIn1, TIn2, TOut>) LambdaMetafactory.metafactory(
                lookup,
                "apply",
                MethodType.methodType(BiFunction.class),
                MethodType.methodType(Object.class, Object.class, Object.class),
                handle,
                MethodType.methodType(returnType, parameterType1, parameterType2)
        ).getTarget().invoke();
    }
}
