package io.gitlab.jfronny.commons.throwable;

import org.jetbrains.annotations.*;

import java.util.Objects;
import java.util.function.Function;

@FunctionalInterface
public interface ThrowingFunction<T, R, TEx extends Throwable> {
    R apply(T var1) throws TEx;

    @Contract(value = "_ -> new", pure = true)
    @ApiStatus.NonExtendable
    default @NotNull ThrowingSupplier<R, TEx> compose(@NotNull ThrowingSupplier<? extends T, ? extends TEx> before) {
        Objects.requireNonNull(before);
        return () -> this.apply(before.get());
    }

    @Contract(value = "_ -> new", pure = true)
    @ApiStatus.NonExtendable
    default <V> @NotNull ThrowingFunction<V, R, TEx> compose(@NotNull ThrowingFunction<? super V, ? extends T, ? extends TEx> before) {
        Objects.requireNonNull(before);
        return (v) -> this.apply(before.apply(v));
    }

    @Contract(value = "_ -> new", pure = true)
    @ApiStatus.NonExtendable
    default <V> @NotNull ThrowingFunction<T, V, TEx> andThen(@NotNull ThrowingFunction<? super R, ? extends V, ? extends TEx> after) {
        Objects.requireNonNull(after);
        return (t) -> after.apply(this.apply(t));
    }

    @Contract(value = "_ -> new", pure = true)
    @ApiStatus.NonExtendable
    default @NotNull ThrowingConsumer<T, TEx> andThen(@NotNull ThrowingConsumer<? super R, ? extends TEx> after) {
        Objects.requireNonNull(after);
        return (t) -> after.accept(this.apply(t));
    }

    @Contract(value = "_ -> new", pure = true)
    @ApiStatus.NonExtendable
    default @NotNull Function<T, R> addHandler(@NotNull Function<Throwable, ? extends R> handler) {
        Objects.requireNonNull(handler);
        return (t) -> {
            try {
                return this.apply(t);
            } catch (Throwable e) {
                return handler.apply(e);
            }
        };
    }

    @Contract(value = "_, _ -> new", pure = true)
    @ApiStatus.NonExtendable
    default @NotNull Function<T, R> addHandler(@NotNull Class<TEx> exception, @NotNull Function<TEx, ? extends R> handler) {
        Objects.requireNonNull(exception);
        Objects.requireNonNull(handler);
        return (t) -> {
            try {
                return this.apply(t);
            } catch (Throwable e) {
                if (exception.isAssignableFrom(e.getClass()))
                    return handler.apply((TEx) e);
                else throw Try.runtimeException(e);
            }
        };
    }

    @Contract(value = "-> new", pure = true)
    @ApiStatus.NonExtendable
    default @NotNull Function<T, R> orThrow() {
        return orThrow(Try::runtimeException)::apply;
    }

    @Contract(value = "_ -> new", pure = true)
    @ApiStatus.NonExtendable
    default <TEx1 extends Throwable> @NotNull ThrowingFunction<T, R, TEx1> orThrow(@NotNull Function<Throwable, TEx1> generator) {
        Objects.requireNonNull(generator);
        return (t) -> {
            try {
                return this.apply(t);
            } catch (Throwable e) {
                throw generator.apply(e);
            }
        };
    }
}
